package resource_model

import (
	"encoding/json"

	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
	"github.com/google/uuid"
)

type AbstractResource struct {
	utils.AbstractObject
	ShortDescription string         `json:"short_description,omitempty" bson:"short_description,omitempty" validate:"required"`
	Description      string         `json:"description,omitempty" bson:"description,omitempty"`
	Logo             string         `json:"logo,omitempty" bson:"logo,omitempty" validate:"required"`
	Owner            string         `json:"owner,omitempty" bson:"owner,omitempty" validate:"required"`
	OwnerLogo        string         `json:"owner_logo,omitempty" bson:"owner_logo,omitempty"`
	SourceUrl        string         `json:"source_url,omitempty" bson:"source_url,omitempty" validate:"required"`
	PeerID           string         `json:"peer_id,omitempty" bson:"peer_id,omitempty" validate:"required"`
	Price            string         `json:"price,omitempty" bson:"price,omitempty"`
	License          string         `json:"license,omitempty" bson:"license,omitempty"`
	ResourceModel    *ResourceModel `json:"resource_model,omitempty" bson:"resource_model,omitempty"`
}

func (abs *AbstractResource) GetModelValue(key string) interface{} {
	if abs.ResourceModel == nil || abs.ResourceModel.Model == nil {
		return nil
	}
	if _, ok := abs.ResourceModel.Model[key]; !ok {
		return nil
	}
	return abs.ResourceModel.Model[key].Value
}

func (abs *AbstractResource) GetModelType(key string) interface{} {
	if abs.ResourceModel == nil || abs.ResourceModel.Model == nil {
		return nil
	}
	if _, ok := abs.ResourceModel.Model[key]; !ok {
		return nil
	}
	return abs.ResourceModel.Model[key].Type
}

func (abs *AbstractResource) GetModelKeys() []string {
	keys := make([]string, 0)
	for k := range abs.ResourceModel.Model {
		keys = append(keys, k)
	}
	return keys
}

func (abs *AbstractResource) GetModelReadOnly(key string) interface{} {
	if abs.ResourceModel == nil || abs.ResourceModel.Model == nil {
		return nil
	}
	if _, ok := abs.ResourceModel.Model[key]; !ok {
		return nil
	}
	return abs.ResourceModel.Model[key].ReadOnly
}

type Model struct {
	Type     string      `json:"type,omitempty" bson:"type,omitempty"`
	Value    interface{} `json:"value,omitempty" bson:"value,omitempty"`
	ReadOnly bool        `json:"readonly,omitempty" bson:"readonly,omitempty"`
}

type ResourceModel struct {
	UUID         string           `json:"id,omitempty" bson:"id,omitempty" validate:"required"`
	ResourceType string           `json:"resource_type,omitempty" bson:"resource_type,omitempty" validate:"required"`
	Model        map[string]Model `json:"model,omitempty" bson:"model,omitempty"`
}

func (ao *ResourceModel) GetID() string {
	return ao.UUID
}

func (r *ResourceModel) GenerateID() {
	r.UUID = uuid.New().String()
}

func (d *ResourceModel) GetName() string {
	return d.UUID
}

func (d *ResourceModel) GetAccessor(caller *tools.HTTPCaller) utils.Accessor {
	data := &ResourceModelMongoAccessor{}
	data.Init(utils.RESOURCE_MODEL, caller)
	return data
}

func (dma *ResourceModel) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *ResourceModel) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}
